import pandas as pd
import numpy as np
import matplotlib.pyplot as plt
import stan
import arviz as az
import scipy.stats as ss

fl_nm = input('Input File(*.xlsx) = ')
with pd.ExcelFile(fl_nm) as f_data:      #   ファイル名の設定
    data_xlsx = pd.read_excel(f_data)    #   DataFrame型への読み込み
print(data_xlsx)                         #   書き出し

trials = data_xlsx['trial']
conds  = data_xlsx['cond']
resps  = data_xlsx['resp']

#
#       U条件のデータの取り出し
#
idx_U = conds == 'U'
trials_U = trials[idx_U]
conds_U = conds[idx_U]
resps_U = resps[idx_U]
#
#       L条件のデータの取り出し
#
idx_L = conds == 'L'
trials_L = trials[idx_L]
conds_L = conds[idx_L]
resps_L = resps[idx_L]
#
#       ヒストグラム
#
plt.hist(resps_U, label = 'U')
plt.hist(resps_L, label = 'L')
plt.plot([350], [0], c='c', marker = 'o', markersize=20, label = '350')
plt.legend()
plt.tight_layout()
plt.savefig('Fig_hist.png')
plt.show()

resps_U = np.array(resps_U)       #     Stan用に型の再設定
resps_L = np.array(resps_L)       #     Stan用に型の再設定

v_mean = np.mean(resps)
v_sd = np.std(resps)

Data = {'mU':len(resps_U), 'ResU':resps_U, 'mL':len(resps_L),
        'ResL':resps_L, 'mean':v_mean, 'sd':v_sd}
with open('model_adj.stan') as f_stan:
    sm = stan.build(f_stan.read(), data = Data)

fit = sm.sample(num_samples = 10000)

i_data = az.from_pystan(posterior = fit, posterior_model = sm)
smry = az.summary(i_data)
print('Summary =\n',smry)
az.plot_trace(i_data, var_names = ('m', 's', 'b'))
plt.tight_layout()
plt.savefig('FigTrace.png')
plt.show()
 

def CalcMAPEst(samples, a = 0.05, n_points = 2000):
    """
        Calculate a MAP estimate from a KDE graph on [Lp, Up]
        Lp and Up are 100*a/2 and 100(1-a/2) percentile points of samples
    """
    import numpy as np
    import scipy.stats as ss
    Lp, Up = np.percentile(samples, [100 * a/2, 100 * (1 - a/2)])  # import numpy as np
    coord = np.linspace(Lp, Up, n_points)
    est_pdf = ss.gaussian_kde(samples).pdf(coord)  # import scipy.stats as ss
    map_idx = np.argmax(est_pdf)
    MAP_Est = coord[map_idx]     
    return MAP_Est, est_pdf[map_idx]

 
dframe =fit.to_frame()
m_paras = dframe['m']
s_paras = dframe['s']
b_paras = dframe['b']

m_map = CalcMAPEst(m_paras)[0]
m_q1, m_q2, m_q3 = np.percentile(m_paras, [25, 50, 75])
print(f'parameter m\nmap = {m_map:.2f}')
print(f'quantiles = {m_q1:.2f}, {m_q2:.2f}, {m_q3:.2f}') 
 
s_map = CalcMAPEst(s_paras)[0]
s_q1, s_q2, s_q3 = np.percentile(s_paras, [25, 50, 75])
print(f'\nparameter s\nmap = {s_map:.2f}')
print(f'quantiles = {s_q1:.2f}, {s_q2:.2f}, {s_q3:.2f}') 

b_map = CalcMAPEst(b_paras)[0]
b_q1, b_q2, b_q3 = np.percentile(b_paras, [25, 50, 75])
print(f'\nparameter b\nmap = {b_map:.2f}')
print(f'quantiles = {b_q1:.2f}, {b_q2:.2f}, {b_q3:.2f}') 

az.plot_kde(m_paras)
plt.title(f'm: map={m_map:.2f}\nq1={m_q1:.3f}, q2={m_q2:.2f}, q3={m_q3:.2f}')
plt.savefig('Fig_m.png')
plt.show()

az.plot_kde(s_paras)
plt.title(f's: map={s_map:.2f}\nq1={s_q1:.3f}, q2={s_q2:.2f}, q3={s_q3:.2f}')
plt.savefig('Fig_s.png')
plt.show()

az.plot_kde(b_paras)
plt.title(f'b: map={b_map:.2f}\nq1={b_q1:.3f}, q2={b_q2:.2f}, q3={b_q3:.2f}')
plt.savefig('Fig_b.png')
plt.show()

x_coord = np.arange(np.min(resps), np.max(resps), 0.01)
U_model = ss.norm.pdf(x_coord, m_map + b_map, s_map)
L_model = ss.norm.pdf(x_coord, m_map - b_map, s_map)
plt.hist(resps_U, density = True, label = 'U')
plt.hist(resps_L, density = True, label = 'L')
plt.plot([350], [0], c='c', marker = 'o', markersize=20, label = '350')
plt.plot(x_coord, U_model, lw = 5, label = 'U-model')
plt.plot(x_coord, L_model, lw = 5, label = 'L-model')
plt.title(f'm = {m_map:.2f},  s = {s_map:.2f},  b = {b_map:.2f}', fontsize = 16)
plt.legend()
plt.tight_layout()
plt.savefig('Fig_distri_model.png')
plt.show()
